#include "scalar.h"

class Vector
{
public:
    Scalar x,y,z;
    Vector(): x(0),y(0),z(0)
    {
    }

    Vector(Scalar _x, Scalar _y, Scalar _z) : x(_x),y(_y),z(_z)
    {
    }

    Vector sum(const Vector& v) const
    {
        return Vector(x + v.x, y + v.y, z + v.z);
    }

    Vector operator +(const Vector& v) const
    {
        return Vector(x + v.x, y + v.y, z + v.z);
    }

    Vector operator *(Scalar a) const
    {
        return Vector(x * a, y * a, z * a);
    }

    // unitary minus (aka flip)
    Vector operator - () const{
        return Vector (-x, -y,-z);
    }

    // binary minus (aka vector difference)
    Vector operator - (const Vector& b) {
        return Vector( x - b.x, y - b.y, z - b.z );
    }

    void print() const;
};


// commutative closure
inline Vector operator *( float k, const Vector& v){
    return v * k;
}

inline Vector mix (const Vector& a, const Vector& b, float t){
    return (1-t)*a + t*b;
}

Scalar dot (const Vector& a, const Vector& b);

Vector cross (const Vector& a, const Vector& b);


